/***************************** FILE HEADER *****************************
#               Copyright (c) 2001 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : sndfile.c
#
#  Module       : Audio Conference Server
#
#  Author(s)    : Agung Suyono  (as1682@columbia.edu)
#                 Chin-Hong Lin (CL486@columbia.edu)
#
#  Created      : 17 Dec 2001
#
#  Description  : Conference Recording for AU, WAV, and rtpdump
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/

#include <stdio.h>
#include <unistd.h>        /* lseek() */
#include <string.h>        /* strcmp() */
#include <strings.h>	   /* bcopy() */ 
#include <unistd.h>        /* open() */
#include <stdlib.h>        /* calloc() */
#include <fcntl.h>         /* O_RDONLY, O_RDWR, O_WRONLY */
#include <sys/types.h>
#include <netinet/in.h>    /* ntohl() */
#include <sys/stat.h>
#include <ctype.h>
#include <wchar.h>
#include "error.h"         /* debug() */
#include "sndfile.h"

/*
* readl16()
* Read 16-bit integer in little endian format.
*/
static u_int16 freadl16(FILE *f)
{
  unsigned char uc1, uc2;
  uc1 = getc(f); 
  uc2 = getc(f);
  return (uc2 << 8) | uc1;
} /* readl16 */


/*
* readl32()
* Read 32-bit integer in little endian format.
*/
static u_int32 freadl32(FILE *f)
{
  unsigned char u[4] = {0,0,0,0};

  if (fread(u, sizeof(char), 4, f) == 4) {
    return ((u_int32)u[3] << 24) | ((u_int32)u[2] << 16) |
           ((u_int32)u[1] <<  8) | ((u_int32)u[0]);
  }
  return 0;
} /* readl32 */

/***********************************
* Create header of WAV and AU file.*
* Success = return FILE pointer;   *
* Fail = NULL			   *
************************************/
FILE *CreateSoundFile(char *filename, encoding_used sndformat) {
  
  FILE *PF;
  char wavbuff[4];
  char aubuff[3];
  char *hdr;
  int lfilename;
  wave_filehdr_t *wavformat;
  audio_filehdr_t *auformat;
  char f_name[1000];
  char *typeloc, *fmt_typeloc, *data_typeloc;

  u_int32 *lengthloc, *wSamplesPerSecloc, *wAvgBytesPerSecloc,
    *magicloc, *hdr_sizeloc, *data_sizeloc, *encodingloc, *sample_rateloc, *channelsloc;
  u_int16 *wFormatTagloc, *wChannelsloc, *wBlockAlignloc, *wBitsPerSampleloc;

  int numwritten;

  strcpy(f_name, filename);

  /* Check for RIFF format */

  if ((strcmp(sndformat.magic, "RIFF")) == 0) {
    /* Check whether it already has ".wav" type */

    lfilename = strlen(f_name);
    wavbuff[4] = '\0';
    wavbuff[3] = toupper(f_name[lfilename-1]);
    wavbuff[2] = toupper(f_name[lfilename - 2]);
    wavbuff[1] = toupper(f_name[lfilename - 3]);
    wavbuff[0] = f_name[lfilename - 4];

    strcpy(f_name, filename);

    if ((strcmp(wavbuff,".WAV")) != 0) {
      debug("CreateSoundFile(LOG)", "Adding string .wav in the end of file name\n");
      //printf("strcmp %d\n", strcmp(wavbuff,".WAV"));
      strcat(f_name, ".wav");
    }
     
    if( (PF = fopen(f_name, "wb+" )) == NULL ) {
      debug("CreateSoundFile(ERROR)", "Write sound file error, EXIT\n");
      return NULL;
    }

    /* Composing RIFF format file header */
    wavformat = calloc(1, sizeof(wave_filehdr_t));

    strcpy(wavformat->magic, "RIFF");
    wavformat->length = 36;
    strcpy(wavformat->type, "WAVE");
    strcpy(wavformat->fmt_chunk.type, "fmt ");
    wavformat->fmt_chunk.length = htonl(16);
    wavformat->fmt_chunk.wFormatTag = sndformat.encoding;
    wavformat->fmt_chunk.wChannels = htonl(sndformat.channel);
    wavformat->fmt_chunk.wSamplesPerSec = sndformat.sample_rate;
    wavformat->fmt_chunk.wBitsPerSample = 8;
    wavformat->fmt_chunk.wAvgBytesPerSec = wavformat->fmt_chunk.wSamplesPerSec * wavformat->fmt_chunk.wChannels * wavformat->fmt_chunk.wBitsPerSample/8;
    wavformat->fmt_chunk.wBlockAlign = wavformat->fmt_chunk.wChannels * wavformat->fmt_chunk.wBitsPerSample/8;
    strcpy(wavformat->data_chunk.type, "data");
    wavformat->data_chunk.length = 0;
      
    hdr = malloc(sizeof(char)*HDRSIZE);

    strcpy(hdr, wavformat->magic);

    lengthloc = (u_int32 *) &hdr[4];
    bcopy(&wavformat->length, lengthloc, sizeof(u_int32));

    typeloc = (char *) &hdr[8];
    bcopy(&wavformat->type, typeloc, sizeof(wavformat->type));

    fmt_typeloc = (char *) &hdr[12];
    bcopy(&wavformat->fmt_chunk.type, fmt_typeloc, sizeof(wavformat->fmt_chunk.type));

    lengthloc = (u_int32 *) &hdr[16];
    bcopy(&wavformat->fmt_chunk.length, lengthloc, sizeof(u_int32));

    wFormatTagloc = (u_int16 *) &hdr[20];
    bcopy(&wavformat->fmt_chunk.wFormatTag, wFormatTagloc, sizeof(u_int16));

    wChannelsloc = (u_int16 *) &hdr[22];
    bcopy(&wavformat->fmt_chunk.wChannels, wChannelsloc, sizeof(u_int16));

    wSamplesPerSecloc = (u_int32 *) &hdr[24];
    bcopy(&wavformat->fmt_chunk.wSamplesPerSec, wSamplesPerSecloc, sizeof(u_int32));

    wAvgBytesPerSecloc = (u_int32 *) &hdr[28];
    bcopy(&wavformat->fmt_chunk.wAvgBytesPerSec, wAvgBytesPerSecloc, sizeof(u_int32));

    wBlockAlignloc = (u_int16 *) &hdr[32];
    bcopy(&wavformat->fmt_chunk.wBlockAlign, wBlockAlignloc, sizeof(u_int16));

    wBitsPerSampleloc = (u_int16 *) &hdr[34];
    bcopy(&wavformat->fmt_chunk.wBitsPerSample, wBitsPerSampleloc, sizeof(u_int16));

    data_typeloc = (char *) &hdr[36];
    bcopy(&wavformat->data_chunk.type, data_typeloc, sizeof(wavformat->data_chunk.type));

    lengthloc = (u_int32 *) &hdr[40];
    bcopy(&wavformat->data_chunk.length, lengthloc, sizeof(u_int32));

    numwritten = fwrite(hdr, sizeof(char), 44, PF);

    debug("CreateSoundFile(LOG)", "Header RIFF file created, file name= %s, size= %d\n", f_name, numwritten);

    return(PF);

  }
   
  /* Check for .au format */

  if ((strcmp(sndformat.magic, ".snd")) == 0) {

    /* Check whether the file ended by ".au" */
    lfilename = strlen(f_name);
    aubuff[3] = '\0';
    aubuff[2] = toupper(f_name[lfilename-1]);
    aubuff[1] = toupper(f_name[lfilename - 2]);
    aubuff[0] = f_name[lfilename - 3];

    if ((strcmp(aubuff, ".AU")) != 0) {
      debug("CreateSoundFile(LOG)", "Adding string .au in the end of file name\n");
      strcat(f_name, ".au");
    }

    if((PF = fopen(f_name, "wb+" )) == NULL ) {
      debug("CreateSoundFile(ERROR)", "Write sound file error, EXIT\n");
      return NULL;
    }

    auformat = calloc(1, sizeof(audio_filehdr_t));

    auformat->magic = AUDIO_FILE_MAGIC;
    auformat->data_size = 0;
    auformat->encoding = sndformat.encoding;
    auformat->sample_rate = sndformat.sample_rate;
    auformat->channels = sndformat.channel;
    auformat->hdr_size = sizeof(auformat);

    hdr = malloc(sizeof(char)*HDRSIZE);

    magicloc = (u_int32 *) &hdr[0];
    bcopy(&auformat->magic, magicloc, sizeof(u_int32));

    hdr_sizeloc = (u_int32 *) &hdr[4];
    bcopy(&auformat->hdr_size, hdr_sizeloc, sizeof(u_int32));

    data_sizeloc = (u_int32 *) &hdr[8];
    bcopy(&auformat->data_size, data_sizeloc, sizeof(u_int32));

    encodingloc = (u_int32 *) &hdr[12];
    bcopy(&auformat->encoding, encodingloc, sizeof(u_int32));

    sample_rateloc = (u_int32 *) &hdr[16];
    bcopy(&auformat->sample_rate, sample_rateloc, sizeof(u_int32));

    channelsloc = (u_int32 *) &hdr[20];
    bcopy(&auformat->channels, channelsloc, sizeof(u_int32));

    numwritten = fwrite(hdr, sizeof(char), 24, PF);

    debug("CreateSoundFile(LOG)", "Header SND file created, file name= %s, size= %d\n", f_name, numwritten);


    return (PF);
  }

  debug("CreateSoundFile(ERROR)", "Unrecognized encoding...\n");
  return NULL;

} /* CreateSoundFile */


/************************
* Close WAV and AU file *
*************************/
void CloseSoundFile(FILE *FN, u_int32 data_size) {
  char magic[4];
  audio_filehdr_t *auformat;
  wave_filehdr_t *wavformat;
  u_int32 tmp_data_size, snd_data_size, file_length;

  tmp_data_size = data_size;

  auformat = calloc(1, sizeof(audio_filehdr_t));
  wavformat = calloc(1, sizeof(wave_filehdr_t));

  fseek(FN, 0, SEEK_SET);
  fread(magic, 4, 1, FN);
  magic[4] = '\0';
  debug("CreateSoundFile(LOG)", "Magic number= %s\n", magic);
 
  if (strcmp(magic, "RIFF") ==0) {

    file_length = tmp_data_size + 36;
    fseek(FN, 0, SEEK_SET);
    fseek(FN, 4, SEEK_SET);
    fwrite(&file_length, sizeof(u_int32), 1, FN);
        
    /*
    fseek(FN, 22, SEEK_SET);
    wavformat->fmt_chunk.wChannels = freadl16(FN);
    printf("wavformat->fmt_chunk.wChannels= %04x\n", wavformat->fmt_chunk.wChannels);

    fseek(FN, 34, SEEK_SET);
    wavformat->fmt_chunk.wBitsPerSample = freadl32(FN);
    printf("wavformat->fmt_chunk.wBitsPerSample %08x\n", wavformat->fmt_chunk.wBitsPerSample);
    */

    //snd_data_size = tmp_data_size * wavformat->fmt_chunk.wChannels * wavformat->fmt_chunk.wBitsPerSample/8;
    snd_data_size = tmp_data_size;
    fseek(FN, 40, SEEK_SET);
    fwrite(&snd_data_size, sizeof(u_int32), 1, FN);

    debug("CloseSoundFile(LOG)", "Written RIFF data size= %ld\n", snd_data_size);

    fseek(FN, 0, SEEK_END);
    fclose(FN);

    return;
  }
  
  if (strcmp(magic, ".snd") ==0) {

    auformat->hdr_size = freadl32(FN);
    
    fwrite(&tmp_data_size, sizeof(u_int32), 1, FN);

    debug("CloseSoundFile(LOG)", "Written AU data size= %ld\n", tmp_data_size);

    
    fclose(FN);
    return;

  }
    
  debug("CloseSoundFile(ERROR)", "Unrecognized file format\n");
  return;

}; /* CloseSoundFile */


/********END OF RECORDING MATERIALS ******/

/* rtp dump recording */


/* 
 * rtpdump_header() 
 * write a header to the current output file called *.rtp
 * The header consists of an identifying string, followed
 * by a binary structure.
*/
void rtpdump_header(FILE *out, struct sockaddr_in *sin,
            struct timeval *start)
{
  RD_hdr_t hdr;

  fprintf(out, "#!rtpplay%s %s/%d\n",RTPFILE_VERSION,
          inet_ntoa(sin->sin_addr), sin->sin_port);
  hdr.start.tv_sec = htonl(start->tv_sec);
  hdr.start.tv_usec = htonl(start->tv_usec);
  hdr.source = sin->sin_addr.s_addr;
  hdr.port = sin->sin_port;
  if(fwrite((char *)&hdr, sizeof(hdr), 1, out) < 1) {
     perror("fwrite");
     exit(1);
   }
} /*rtpdump_header*/


/* 
 * parse_header() 
 * Return header length of RTP packet contained in 'buf'.
*/
int parse_header(char *buf)
{
  v_rtp_hdr_t *r = (v_rtp_hdr_t *)buf;
  int hlen = 0;

  hlen = 12 + r->cc * 4;

  return hlen;
} // parse_header 

/*
 * packet_handler()
 * Process one packet and write it to file 'out'
*/
void packet_handler(FILE *out, int trunc,
  double dstart, struct timeval now, int ctrl, 
  struct sockaddr_in sin, int len, RD_buffer_t
  packet)
{
  double dnow = tdbl(&now);
  int hlen;   /* header length */
  int offset;

  hlen = ctrl ? len : parse_header(packet.p.data);
  offset = (dnow - dstart) * 1000;
  packet.p.hdr.offset = htonl(offset);
  packet.p.hdr.plen   = ctrl ? 0 : htons(len);
  /* truncation of payload */
  if (!ctrl && (len - hlen > trunc)) len = hlen + trunc;
  packet.p.hdr.length = htons(len + sizeof(packet.p.hdr));
  if (fwrite((char *)&packet, len + sizeof(packet.p.hdr), 1, out) == 0) {
  perror("fwrite");
  exit(1);
   }
} /* packet_handler */

/*
 * tdbl()
 * Convert timeval 'a' to double.
*/
double tdbl(struct timeval *a)
{
  return a->tv_sec + a->tv_usec/1e6;
}  /* tdbl() */







