/***************************** FILE HEADER *****************************
#               Copyright (c) 2001 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : sndfile.h
#
#  Module       : Audio Conference Server
#
#  Author(s)    : Agung Suyono (as1682@columbia.edu)
#                 Chin-Hong Lin(CL486@columbia.edu)
#
#  Created      : 17 Dec 2001
#
#  Description  : Prototypes for Conference Recording 
#                
#  Notes        : 
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/


#ifndef SNDFILE_H
#define SNDFILE_H

#include <sys/socket.h>	/* sockadd_in */
#include <sys/time.h>   /* timeval */
#include <string.h>	/* bcopy() */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "../librtp/rtp_api.h"	/* u_int8, u_int16, u_int32 */

/* Wave format */
#define WAVE_FORMAT_UNKNOWN     (0x0000)
#define WAVE_FORMAT_PCM         (0x0001)
#define WAVE_FORMAT_ADPCM       (0x0002)
#define WAVE_FORMAT_ALAW        (0x0006)
#define WAVE_FORMAT_MULAW       (0x0007)
#define WAVE_FORMAT_OKI_ADPCM   (0x0010)
#define WAVE_FORMAT_DVI_ADPCM   (0x0011)
#define WAVE_FORMAT_DIGISTD     (0x0015)
#define WAVE_FORMAT_DIGIFIX     (0x0016)
#define IBM_FORMAT_MULAW        (0x0101)
#define IBM_FORMAT_ALAW         (0x0102)
#define IBM_FORMAT_ADPCM        (0x0103)

#define AUDIO_FILE_MAGIC 0x2e736e64   /* .snd */
/* Define the encoding fields */
#define AUDIO_FILE_ENCODING_MULAW_8       (1) /* 8-bit ISDN u-law */
#define AUDIO_FILE_ENCODING_LINEAR_8      (2) /* 8-bit linear PCM */
#define AUDIO_FILE_ENCODING_LINEAR_16     (3) /* 16-bit linear PCM */
#define AUDIO_FILE_ENCODING_LINEAR_24     (4) /* 24-bit linear PCM */
#define AUDIO_FILE_ENCODING_LINEAR_32     (5) /* 32-bit linear PCM */
#define AUDIO_FILE_ENCODING_FLOAT         (6) /* 32-bit IEEE floating point */
#define AUDIO_FILE_ENCODING_DOUBLE        (7) /* 64-bit IEEE floating point */
#define AUDIO_FILE_ENCODING_ADPCM_G721   (23) /* 4-bit CCITT g.721 ADPCM */
#define AUDIO_FILE_ENCODING_ADPCM_G722   (24) /* CCITT g.722 ADPCM */
#define AUDIO_FILE_ENCODING_ADPCM_G723_3 (25) /* CCITT g.723 3-bit ADPCM */
#define AUDIO_FILE_ENCODING_ADPCM_G723_5 (26) /* CCITT g.723 5-bit ADPCM */
#define AUDIO_FILE_ENCODING_ALAW_8       (27) /* 8-bit ISDN A-law */

#define HDRSIZE 4096

#define RTPFILE_VERSION "1.0"
#define RTP_VERSION    2
#ifdef WORDS_BIGENDIAN
#define RTP_BIG_ENDIAN 1
#else
#define RTP_LITTLE_ENDIAN 1
#endif


typedef struct {
  u_int32 magic;          /* magic number */
  u_int32 hdr_size;       /* size of this header, with info (in bytes) */
  u_int32 data_size;      /* length of data (optional) */
  u_int32 encoding;       /* data encoding format */
  u_int32 sample_rate;    /* samples per second */
  u_int32 channels;       /* number of interleaved channels */
} audio_filehdr_t;


typedef struct {
  char magic[4]; /*= "RIFF";    magic constant */
  u_int32 length;           /* total length of file - 8 */
  char type[4]; /*  = "WAVE";    designates as WAVE file */
  struct {
    char type[4]; /* = "fmt ";       type of chunk */
    u_int32 length;             /* length in bytes */
    u_int16 wFormatTag;         /* data format */
    u_int16 wChannels;          /* number of channels */
    u_int32 wSamplesPerSec;     /* samples per second per channel */
    u_int32 wAvgBytesPerSec;    /* estimate of bytes per second */
    u_int16 wBlockAlign;        /* byte alignment of a basic sample */
    u_int16 wBitsPerSample;     /* bits per sample */
  } fmt_chunk;
  struct {
    char type[4]; /* = "data";       type of chunk */
    u_int32 length;		  /* length of the data (chunk size minus (-) 8 bytes */
  } data_chunk;
} wave_filehdr_t;


typedef struct {
  char magic[4];
  char type[4];
  int encoding;
  int sample_rate;
  int channel;
} encoding_used;

typedef u_int32 member_t;

typedef enum {
    RTCP_SDES_END   = 0,
    RTCP_SDES_CNAME = 1,
    RTCP_SDES_NAME  = 2,
    RTCP_SDES_EMAIL = 3,
    RTCP_SDES_PHONE = 4,
    RTCP_SDES_LOC   = 5,
    RTCP_SDES_TOOL  = 6,
    RTCP_SDES_NOTE  = 7,
    RTCP_SDES_PRIV  = 8
} rtcp_sdes_type_t;

typedef struct {
  struct timeval start;  /* start of recording (GMT) */
  u_int32 source;        /* network source (multicast address) */
  u_int16 port;          /* UDP port */
} RD_hdr_t;

typedef struct {
  u_int16 length;    /* length of packet, including this header (may 
                        be smaller than plen if not whole packet recorded) */
  u_int16 plen;      /* actual header+payload length for RTP, 0 for RTCP */
  u_int32 offset;    /* milliseconds since the start of recording */
} RD_packet_t;

typedef union {
  struct {
    RD_packet_t hdr;
    char data[8000];
  } p;
  char byte[8192];
} RD_buffer_t;

/*
 * RTCP common header word
 */
typedef struct {
#if RTP_BIG_ENDIAN
    unsigned int version:2;   /* protocol version */
    unsigned int p:1;         /* padding flag */
    unsigned int count:5;     /* varies by packet type */
#elif RTP_LITTLE_ENDIAN
    unsigned int count:5;     /* varies by packet type */
    unsigned int p:1;         /* padding flag */
    unsigned int version:2;   /* protocol version */
#else
#error Define one of RTP_LITTLE_ENDIAN or RTP_BIG_ENDIAN
#endif
    unsigned int pt:8;        /* RTCP packet type */
    unsigned int length:16;   /* pkt len in words, w/o this word */
} rtcp_common_t;

/*
 * Reception report block
 */
typedef struct {
    u_int32 ssrc;             /* data source being reported */
    unsigned int fraction:8;  /* fraction lost since last SR/RR */
    int lost:24;              /* cumul. no. pkts lost (signed!) */
    u_int32 last_seq;         /* extended last seq. no. received */
    u_int32 jitter;           /* interarrival jitter */
    u_int32 lsr;              /* last SR packet from this source */
    u_int32 dlsr;             /* delay since last SR packet */
} rtcp_rr_t;

/*
 * SDES item
 */
typedef struct {
    u_int8 type;              /* type of item (rtcp_sdes_type_t) */
    u_int8 length;            /* length of item (in octets) */
    char data[1];             /* text, not null-terminated */
} rtcp_sdes_item_t;


/*
 * RTP data header
 */
typedef struct {
#if RTP_BIG_ENDIAN
    unsigned int version:2;   /* protocol version */
    unsigned int p:1;         /* padding flag */
    unsigned int x:1;         /* header extension flag */
    unsigned int cc:4;        /* CSRC count */
    unsigned int m:1;         /* marker bit */
    unsigned int pt:7;        /* payload type */
#elif RTP_LITTLE_ENDIAN
    unsigned int cc:4;        /* CSRC count */
    unsigned int x:1;         /* header extension flag */
    unsigned int p:1;         /* padding flag */
    unsigned int version:2;   /* protocol version */
    unsigned int pt:7;        /* payload type */
    unsigned int m:1;         /* marker bit */
#else
#error Define one of RTP_LITTLE_ENDIAN or RTP_BIG_ENDIAN
#endif
    unsigned int seq:16;      /* sequence number */
    u_int32 ts;               /* timestamp */
    u_int32 ssrc;             /* synchronization source */
    u_int32 csrc[1];          /* optional CSRC list */
} v_rtp_hdr_t;

/*
 * One RTCP packet
 */
typedef struct {
    rtcp_common_t common;     /* common header */
    union {
        /* sender report (SR) */
        struct {
            u_int32 ssrc;     /* sender generating this report */
            u_int32 ntp_sec;  /* NTP timestamp */
            u_int32 ntp_frac;
            u_int32 rtp_ts;   /* RTP timestamp */
            u_int32 psent;    /* packets sent */
            u_int32 osent;    /* octets sent */ 
            rtcp_rr_t rr[1];  /* variable-length list */
        } sr;

        /* reception report (RR) */
        struct {
            u_int32 ssrc;     /* receiver generating this report */
            rtcp_rr_t rr[1];  /* variable-length list */
        } rr;

        /* source description (SDES) */
        struct rtcp_sdes {
            u_int32 src;      /* first SSRC/CSRC */
            rtcp_sdes_item_t item[1]; /* list of SDES items */
        } sdes;

        /* BYE */
        struct {
            u_int32 src[1];   /* list of sources */
            /* can't express trailing text for reason */
        } bye;
    } r;
} rtcp_t;


/*** the following two are for AU and WAV format recording ****/
extern FILE *CreateSoundFile(char *filename, encoding_used sndformat); 
extern void CloseSoundFile(FILE *FN, u_int32 data_size); 


/*** the following are for rtpdump recording ****/
extern double tdbl(struct timeval *a);
extern void rtpdump_header(FILE *out, struct sockaddr_in *sin,
                           struct timeval *start);
extern int parse_header(char *buf);
extern void packet_handler(FILE *out, int trunc,
            double dstart, struct timeval now, int ctrl, 
            struct sockaddr_in sin, int len, RD_buffer_t packet);


#endif

/******************END of FILE **************************************/

