/***************************** FILE HEADER *****************************
#               Copyright (c) 2001 by Columbia University              *
#                        All Rights Reserved                           *
#***********************************************************************
#***********************************************************************
#
#  File Name    : rf.c
#
#  Module       : rtspd recording support
#
#  Author(s)    : Agung Suyono  (as1682@columbia.edu)
#                 Chin-Hong Lin (CL486@columbia.edu)
#
#  Created      : 17 Dec 2001
#
#  Description  : rtspd recording using rtpdump
#                
#  Notes        : codes are addopted from 
#                 http://www.cs.columbia.edu/~hgs/software/rtptools/src
#
#***********************************************************************
#                Modification History (Reverse Order)
#-----------------------------------------------------------------------
#   Date         Author              Description
#-----------------------------------------------------------------------
#              |             | 
#              |             | 
#-----------------------------------------------------------------------
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>  /* struct sockaddr_in */
#include <arpa/inet.h>   /* inet_ntoa() */
#include <sys/select.h>
#include <sys/time.h>    /* gettimeofday() */
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>      /* select(), perror() */
#include <stdlib.h>      /* getopt(), atoi() */
#include <math.h>        /* fmod() */
#include <fcntl.h>       /* O_BINARY on Win32 */
#include "dstring.h"
#include "rtp_pt.h"
#include "error.h"
#include "rf.h"

#define RTPFILE_VERSION "1.0"
#define RTP_VERSION 2

/*
* Read header. Return -1 if not valid, 0 if ok.
*/
int RD_header(FILE *in, struct sockaddr_in *sin, int verbose)
{
  RD_hdr_t hdr;
  char line[80], magic[80];

  if (fgets(line, sizeof(line), in) == NULL) return -1;
  sprintf(magic, "#!rtpplay%s ", RTPFILE_VERSION);
  if (strncmp(line, magic, strlen(magic)) != 0) return -1;
  if (fread((char *)&hdr, sizeof(hdr), 1, in) == 0) return -1;
  hdr.start.tv_sec = ntohl(hdr.start.tv_sec);
  hdr.port         = ntohs(hdr.port);
  if (verbose) {
    struct tm *tm;
    struct in_addr in;

    in.s_addr = hdr.source;
    tm = localtime(&hdr.start.tv_sec);
    strftime(line, sizeof(line), "%C", tm);
    printf("Start:  %s\n", line);
    printf("Source: %s (%d)\n", inet_ntoa(in), hdr.port);
  }
  if (sin && sin->sin_addr.s_addr == 0) {
    sin->sin_addr.s_addr = hdr.source;
    sin->sin_port        = htons(hdr.port);
  }
  return 0;
} /* RD_header */

/*
* Read next record from input file.
*/
int RD_read(FILE *in, RD_buffer_t *b)
{
  /* read packet header from file */
  if (fread((char *)b->byte, sizeof(b->p.hdr), 1, in) == 0) {
    /* we are done */
    return 0;
  }

  /* convert to host byte order */
  b->p.hdr.length = ntohs(b->p.hdr.length) - sizeof(b->p.hdr);
  b->p.hdr.offset = ntohl(b->p.hdr.offset);
  b->p.hdr.plen   = ntohs(b->p.hdr.plen);

  /* read actual packet */
  if (fread(b->p.data, b->p.hdr.length, 1, in) == 0) {
    perror("fread body");
  } 
  return b->p.hdr.length; 
} /* RD_read */

/*
* Convert timeval 'a' to double.
*/
double tdbl(struct timeval *a)
{
  return a->tv_sec + a->tv_usec/1e6;
}

/*
* Write a header to the current output file.
* The header consists of an identifying string, followed
* by a binary structure.
*/
void rtpdump_header(FILE *out, struct sockaddr_in *sin,
  struct timeval *start)
{
  RD_hdr_t hdr;

  fprintf(out, "#!rtpplay%s %s/%d\n", RTPFILE_VERSION,
    inet_ntoa(sin->sin_addr), sin->sin_port);
  hdr.start.tv_sec  = htonl(start->tv_sec);
  hdr.start.tv_usec = htonl(start->tv_usec);
  hdr.source = sin->sin_addr.s_addr;
  hdr.port   = sin->sin_port;
  if (fwrite((char *)&hdr, sizeof(hdr), 1, out) < 1) {
    perror("fwrite");
    exit(1);
  } 
} /* rtpdump_header */


/*
* Return header length of RTP packet contained in 'buf'.
*/
static int parse_header(char *buf)
{
  v_rtp_hdr_t *r = (v_rtp_hdr_t *)buf;
  int hlen = 0;

  if (r->version == 0) {
    vat_hdr_t *v = (vat_hdr_t *)buf;
    hlen = 8 + v->nsid * 4;
  }
  else if (r->version == RTP_VERSION) {
    hlen = 12 + r->cc * 4;
  }

  return hlen;
} /* parse_header */



/*
 * packet_handler()
 * Process one packet and write it to file 'out'
*/
void packet_handler(FILE *out, t_format format, int trunc,
  double dstart, struct timeval now, int ctrl, 
  struct sockaddr_in sin, int len, RD_buffer_t
  packet)
{
  double dnow = tdbl(&now);
  int hlen;   /* header length */
  int offset;

    
  hlen = ctrl ? len : parse_header(packet.p.data);
  offset = (dnow - dstart) * 1000;
  packet.p.hdr.offset = htonl(offset);
  packet.p.hdr.plen   = ctrl ? 0 : htons(len);
  /* truncation of payload */
  if (!ctrl && (len - hlen > trunc)) len = hlen + trunc;
  packet.p.hdr.length = htons(len + sizeof(packet.p.hdr));
  if (fwrite((char *)&packet, len + sizeof(packet.p.hdr), 1, out) == 0) {
    perror("fwrite");
    exit(1);
  }
      
} /* packet_handler */


