/*
* Read and write RTP (.rtp) files.
*/

#include <stdio.h>         /* SEEK_CUR and SEEK_SET */
#include "sysdep.h"
#include "dstring.h"
#include "rtp_pt.h"
#include <unistd.h>        /* lseek() */
#include <string.h>        /* strcmp() */
#include <unistd.h>        /* open() */
#include <stdlib.h>        /* calloc() */
#include <fcntl.h>         /* O_RDONLY, O_RDWR, O_WRONLY */
#include <sys/types.h>
#include <netinet/in.h>    /* ntohl() */
#include <sys/stat.h>
#include "error.h"         /* debug () */
#include "getusername.h"   /* getusername () */
#include "rf.h"

#define MAX_FORMATS 100

/* Hapus kalau payload type-nya sudah availabe */
#define RTP 1
/* batas hapus */

/*
typedef enum {F_invalid, F_dump, F_header, F_hex, F_rtcp, F_short,
   F_payload, F_ascii} t_format;
*/
typedef struct {
  FILE * fp; /* File */
  int rw; /* Mode. O_RDONLY, O_WRONLY */
  long fp_start; /* the start position read/write in the file */
  double ts_start;  /* timestamp of starting the recording/reading */
  int prev_ts;    /* previous timestamp, for computing tsinc in RtpRead */
} RtpFile;

typedef struct {
  int pt; /* Payload type */
  int tsinc; /* Timestamp increment from previous packet. */
  int seq;   /* Sequence number */
  double npt; /* npt timestamp */
  int length; /* Length ofdata */
  char buffer[1500]; /* buffer */

} RtpFilePacket;

static struct sockaddr_in sin;
static struct timeval start;
static double dstart;

int GetNextRtpPacket(FILE *rtp, RtpFilePacket *packet ) 
{
  if ( fread(&packet->pt, 4, 1, rtp)<=0 ||
       fread(&packet->tsinc,4,1,rtp) <= 0 ||
       fread(&packet->seq,4,1,rtp) <= 0 ||
       fread(&packet->npt,sizeof(packet->npt),1,rtp) <= 0 ||
       fread(&packet->length,4,1,rtp) <= 0 ) {
    return -1;
  }

  if ( packet->length > 0 ) {
    if ( fread(packet->buffer, 1, packet->length > sizeof(packet->buffer) ?
               sizeof(packet->buffer) : packet->length, rtp) <=0)
      {
        return -1;
      }
  }

  packet->pt = ntohl(packet->pt);
  packet->tsinc = ntohl(packet->tsinc);
  packet->seq = ntohl(packet->seq);
  packet->length = ntohl(packet->length);

  return 0;
}

double RtpSeek(RtpFile *rtp, double npt);

RtpFile *RtpOpen(char *path, char *track, int rw)
{
  char *mode;
  RtpFile *rtp;           /* rtp file object */

  rtp = calloc(1, sizeof(RtpFile));

  if ( rw == O_RDONLY ) {
    mode = "rb";
  }
  else if ( rw == O_WRONLY) {
    mode = "wb";
  }
  else {
    mode = "rb+";
  }

  if ((rtp->fp = fopen(path, mode)) < 0) {
    debug("RtpOpen", "Could not open file %s in mode %s\n", path, mode);
    free(rtp);
    return NULL;
  }

  /*initialize the address/port receiving RTP/RTCP in rtspd */
  memset(&sin,0,sizeof sin);
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  sin.sin_port = RTP_PORT;  //not sure now

  gettimeofday(&start, 0);
  dstart = tdbl(&start);
  rtp->ts_start = dstart;

  rtp->rw = rw;

  /* Set previous timestamp to 0 */
  rtp->prev_ts = 0;

  if ( rw == O_RDONLY ) {

    /* read header of input file */
    if (RD_header(rtp->fp, &sin, 0) < 0) {
      fprintf(stderr, "Invalid header\n");
      exit(1);
    }

  }
  else if (rw == O_WRONLY) {
   
    /* Initialize rtpdump header */
    rtpdump_header(rtp->fp, &sin, &start);

  }

  rtp->fp_start = ftell(rtp->fp);
  if ( rtp->fp_start < 0 ) {
    error("RtpOpen", "ftell() failed\n");
    fclose(rtp->fp);
    free(rtp);
    return NULL;
  }

  return rtp;

} /* RtpOpen */


/*
* RtpSeek()
* Position within sound file to given time value.  Return actual
* position, accounting for any block structure in file, if successful,
* -1 on error. Return duration if
* npt is negative.
*/
double RtpSeek(RtpFile *rtp, double npt)
{
  long lastpos;
  RD_buffer_t packet;


  //RtpFilePacket packet;

  if ( rtp == NULL ) {
    return 0.0;
  }

  fseek(rtp->fp, rtp->fp_start, SEEK_SET);

  if ( npt == 0. ) {
    /* If start of the file is requested retrun now. */
    return 0.;
  }

  /* Don't know yet how to find the duration */
  if ( npt < 0.0 ) {
    return packet.p.hdr.offset; /* duration */
  }

  /* Get next packet; try again if we haven't reach the requested npt */
  do {
    if (RD_read(rtp->fp, &packet) == 0) {
      /* No packet available */
      return 0.;
    }
  } while (packet.p.hdr.offset < npt);

  lastpos = ftell(rtp->fp);
  fseek(rtp->fp, lastpos, SEEK_SET);

  return packet.p.hdr.offset;
} /* RtpSeek */

/*
* RtpDescribe()
* Return a description of a sound file. Return NULL if the type is not
* understood.
*/
char *RtpDescribe(RtpFile *rtp, char *type)
{
  DString d;
  char username[USERNAME_MAX];
  int audio_formats[MAX_FORMATS+1];
  int i;
  long pos;
  //RtpFilePacket  packet;
  RD_buffer_t packet;
  double npt;
  v_rtp_hdr_t *r;

  if ( rtp == NULL || rtp->rw != O_RDONLY) {
    debug("RtpDescribe", "RTP is null or mode is not readonly\n");
    return NULL;
  }

  audio_formats[0] = -1;

  /* Get the file size */
  npt =  RtpSeek(rtp, -1.);

  pos = ftell(rtp->fp);

  fseek(rtp->fp, rtp->fp_start, SEEK_SET);
  while( RD_read(rtp->fp, &packet) > 0 ) {
    int oFound = 0;
    int j;

    r = (v_rtp_hdr_t *)packet.p.data;
    int pt = r->pt;
    for ( j=0; audio_formats[j] >= 0 && j < MAX_FORMATS; j++ ) {
      if ( audio_formats[j] == pt ) {
        oFound = 1;
      }
    }
    if ( !oFound ) {
      audio_formats[j] = pt;
      audio_formats[j+1] = -1;
    }
  }

  fseek(rtp->fp, pos, SEEK_SET);

  getusername(username, sizeof(username));

  DStringInit(&d);
  if (!strcmp(type, "application/sdp")) {
    int oFound = 0;

    DStringPrintf(&d, "v=0\r\n");
    DStringPrintf(&d, "o=%s %lu %lu IN IP4 %s\r\n",
      username[0] != '\0' ? username : "-", (long) time(0), (long) time(0), 
      "127.0.0.1");
    /* s= line must be present */
    DStringPrintf(&d, "s=(no subject)\r\n");
    DStringPrintf(&d, "t=0 0\r\n");

    DStringPrintf(&d, "a=length:npt=%.3f\r\n", npt);
    DStringPrintf(&d, "m=audio %d RTP/AVP", 0);
    for ( i=0; audio_formats[i] >= 0 && i < MAX_FORMATS; i++) {
      DStringPrintf(&d, " %d", audio_formats[i]);
      debug("RtpDescribe", "Adding format %d\n", audio_formats[i]);
      oFound = 1;
    }
    if ( !oFound ) {
      DStringPrintf(&d, " 0");
    }
    DStringAppend(&d, "\r\n", -1);
  }
  return d.string;
} /* SndDescribe */


/*
* RtpRead()
* Read a block of snd data into 'buffer' and determine next playout
* time.  Set 'pt' to RTP payload type and 'tsinc' to time stamp
* increment.  Return number of bytes read.  Return 0 on EOF and -1 on
* error.
*/
int RtpRead(RtpFile *rtp, char *buffer, double *npt, int *pt, int *tsinc)
{
  v_rtp_hdr_t *r;
  RD_buffer_t packet;

  if ( rtp == NULL || rtp->rw != O_RDONLY) {
    debug("RtpRead", "Null or not read mode\n");
    return -1;
  }

  if (RD_read(rtp->fp, &packet) <= 0) {
    debug("RtpRead", "RD_read returned -ve\n");
    return 0;
  }

  r = (v_rtp_hdr_t *)packet.p.data;
  
  //gettimeofday(&now, NULL);
  if (npt) {
    *npt = packet.p.hdr.offset; 
  }

  if (pt) {
    *pt = r->pt;
  }
  
  if (tsinc) {
    *tsinc = r->ts - rtp->prev_ts;
  }
  

  if (buffer && packet.p.hdr.length > 0) {
    memcpy(buffer, packet.p.data, packet.p.hdr.length);
  }

  /* Return length */
  return packet.p.hdr.length;

} /* RtpRead */



/*
* RtpWrite()
* Write a block of snd data.  Return number of bytes written.  Return 0
* on EOF and -1 on error.
*/
int RtpWrite(RtpFile *rtp, int pt, int ts, int seq, char *buffer,
  int size, double *npt)
{
  t_format format = F_dump;
  int trunc = 1000000;  /* bytes to show for F_dump */
  struct sockaddr_in mysin;
  struct timeval now;
  double dnow;
  RD_buffer_t packet;
  int i;
  //v_rtp_hdr_t *r = (v_rtp_hdr_t *)buffer;
  char *pt_loc;
  char pt;

  if (rtp == NULL || rtp->rw != O_WRONLY) {
    debug("RtpWrite", "Null or not write mode\n");
    return -1;
  }

  /* We don't need sockaddr_in in rtpdump so we set it arbitrarily */
  memset(&sin,0,sizeof sin);
  mysin.sin_family = AF_INET;
  mysin.sin_addr.s_addr = INADDR_ANY;
  mysin.sin_port = 0;  

  gettimeofday(&now, NULL);
  dnow = tdbl(&now);
  dstart = rtp->ts_start;    

  memcpy(&(packet.p.data), buffer, sizeof(buffer));

  /* Check whether it's RTCP packet */
  pt_loc = (char *) &buffer[1];
  pt = *pt_loc;

  if ((pt == (char)200) || (pt == (char)201) || (pt == (char)202) || (pt == (char)203) || (pt == (char)204)) {
    i = 1;	// RTCP Packet
    packet_handler(rtp->fp, format, trunc, dstart, now, i, mysin, 
      sizeof(buffer), packet);
  } else {	// RTP Packet
    i = 0;
    packet_handler(rtp->fp, format, trunc, dstart, now, i, mysin, 
      sizeof(buffer), packet);
  }


  return sizeof(buffer);

} /* RtpWrite */


/*
* RtpClose()
* Close the file and release all resources.
*/
int RtpClose(RtpFile *rtp)
{
  if ( rtp == NULL ) {
    return -1;
  }

  fclose(rtp->fp);

  free(rtp);

  return 0;
} /* RtpClose */



